<?php
namespace Teknisa\OPL\Modules\Consolidacao\Controllers;

use Teknisa\GER\Controllers\TeknisaCrud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Response\Error;

class Opl00100LoteDeEnvioDePedidosDoOLController extends TeknisaCrud {
    
    protected $crudService;
	protected $dataSourceManager;
	protected $parameterBag;
	protected $environment;
    protected $dataSourceName;
    protected $parameterBagColumns = array();
    protected $newCode;
    
    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, $newCode) {
        parent::__construct($crudService, $parameterBag);
        
        $this->dataSourceManager    = $dataSourceManager;
        $this->environment          = $environment;
		$this->newCode              = $newCode;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        
        $P_CDLOTENVITOL = null;
        
        foreach($conditions as $condition) {
            if($condition["columnName"] == "TCDLOTENVITOL") {
                if($condition["value"] == 'T') {
                    $P_CDLOTENVITOL = 'T';
                }
            } else if($condition["columnName"] == "P_CDLOTENVITOL") {
                if(is_array($condition["value"])) {
                    if(count($condition["value"]) != 0) {
                        $loteStr = '_';
                        foreach($condition["value"] as $lote) {
                            $loteStr .= $lote . '_';
                        }
                        $P_CDLOTENVITOL = $loteStr;
                    } else {
                        $P_CDLOTENVITOL = 'T';
                    }
                } else {
                    $P_CDLOTENVITOL = 'T';
                }
            }
        }
        
        $filter = array(
            array(
                "columnName"=>"P_CDLOTENVITOL",
                "value"=>$P_CDLOTENVITOL,
                "operator"=>"="
            )
        );
        
        $parameterBagColumns = array("P_CDLOTENVITOL");
        
        $this->dataSourceName = "opl00100_lote";
        
        $newFilterCriteria = new FilterCriteria($this->dataSourceName, $filter);
        $newRequest = new Filter($newFilterCriteria, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
    
        parent::find($newRequest, $response);
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = "opl00100_update";
        
        try {
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function delete(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = "opl00100_update";
        
        try {
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getNewLoteCode(DTO\Request $request, DTO\Response $response) {
        
        $novoCodigo = $this->newCode->getCodeByTableName("LOTENVITMEXOL", $this->environment->getNrOrg());
        
        $response->addDataSet(new DataSet("opl00100_newCode", array($novoCodigo)));
    }
}