<?php
/**
 * Suprimentos\Logic\Cotacao
 *
 * @category Php
 * @package  Suprimentos\Logic\Cotacao
 * @author   matthaeuscarvalho <matthaeus.carvalho@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfoodcomp
 */
namespace Suprimentos\Logic\Cotacao;

use Fiscal\Entities\Aliquota;
use Geral\Entities\Fornecedor;
use Geral\Entities\Parasupri;
use Geral\Entities\Prazopgtforn;
use Geral\Entities\Produto;
use Geral\Util\DateUtil;
use Geral\Util\NovoCodigo;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;
use Setforms2\Shared\Registry;
use Suprimentos\Entities\Itemcole;
use Suprimentos\Entities\Coletapr;
use Suprimentos\Entities\Pzopgtitmcol;
use Suprimentos\Entities\Prodcomp;
use Suprimentos\Logic\Comprador\CompradorLogic;
use Suprimentos\Logic\Exception\ExceptionCotacao;

/**
 * Class Cotacao
 *
 * @category Php
 * @package  Suprimentos\Logic\Cotacao
 * @author   matthaeuscarvalho <matthaeus.carvalho@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class Cotacao
{
    /**
     * Executa Procedure IncluiItemcolAcomprar
     *
     * @param String $cdfilial    Filial
     * @param String $cdfiliproc  Filial Proc
     * @param String $nrcolepreco Número da Coleta de Preço
     * @param String $dtini       Data Inicial
     * @param String $dtfin       Data Final
     * @param String $prodini     Produto Inicial
     * @param String $prodfin     Produto Final
     * @param String $cdregiao    Região
     * @param String $cdforneced  Fornecedor
     * @param String $cdmarca     Marca
     * @param String $consist     Consist
     * @param String $idultprec   Carregar ùltimo preço
     * @param String $cdoperincl  Operador de Inclusão
     *
     * @throws \Exception
     * @return null
     */
    public static function execProcedureIncluiItemcolAcomprar(
        $cdfilial, $cdfiliproc, $nrcolepreco, $dtini, $dtfin, $prodini, $prodfin, $cdregiao, $cdforneced, $cdmarca,
        $consist, $idultprec, $cdoperincl
    ) {
        try {
            $procedure = new Procedure('INCLUI_ITEMCOL_ACOMPRAR');
            $procedure->addParam(
                new Param(
                    'P_CDFILIAL',
                    Param::PARAM_INPUT,
                    $cdfilial,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDFILIPROC',
                    Param::PARAM_INPUT,
                    $cdfiliproc,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_NRCOLEPRECO',
                    Param::PARAM_INPUT,
                    $nrcolepreco,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_DTINI',
                    Param::PARAM_INPUT,
                    $dtini,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_DTFIN',
                    Param::PARAM_INPUT,
                    $dtfin,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_PRODINI',
                    Param::PARAM_INPUT,
                    $prodini,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_PRODFIN',
                    Param::PARAM_INPUT,
                    $prodfin,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDREGIAO',
                    Param::PARAM_INPUT,
                    $cdregiao,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDFORNECED',
                    Param::PARAM_INPUT,
                    $cdforneced,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDMARCA',
                    Param::PARAM_INPUT,
                    $cdmarca,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CONSIST',
                    Param::PARAM_INPUT,
                    $consist,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_IDULTPREC',
                    Param::PARAM_INPUT,
                    $idultprec,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDOPERINCL',
                    Param::PARAM_INPUT,
                    $cdoperincl,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_ROWSAFFECT',
                    Param::PARAM_OUTPUT,
                    null,
                    Param::PARAM_TYPE_STR,
                    200
                )
            );

            $result = $procedure->call();
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Executa Procedure IncluiItemcolTodos
     *
     * @param String $cdfilial    Filial
     * @param String $cdfiliproc  Filial Proc
     * @param String $nrcolepreco Número Coleta de Preço
     * @param String $dtini       Data Inicial
     * @param String $dtfin       Data Final
     * @param String $prodini     Produto Inicial
     * @param String $prodfin     Produto Final
     * @param String $cdregiao    Região
     * @param String $cdforneced  Fornecedor
     * @param String $cdmarca     Marca
     * @param String $consist     Consist
     * @param String $idultprec   Carregar Último Preço
     * @param String $cdoperincl  Operador de Inclusão
     *
     * @throws \Exception
     * @return null
     */
    public static function execProcedureIncluiItemcolTodos(
        $cdfilial, $cdfiliproc, $nrcolepreco, $dtini, $dtfin, $prodini, $prodfin,
        $cdregiao, $cdforneced, $cdmarca,
        $consist, $idultprec, $cdoperincl
    ) {
        try {
            $procedure = new Procedure('INCLUI_ITEMCOL_TODOS');
            $procedure->addParam(
                new Param(
                    'P_CDFILIAL',
                    Param::PARAM_INPUT,
                    $cdfilial,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDFILIPROC',
                    Param::PARAM_INPUT,
                    $cdfiliproc,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_NRCOLEPRECO',
                    Param::PARAM_INPUT,
                    $nrcolepreco,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_DTINI',
                    Param::PARAM_INPUT,
                    $dtini,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_DTFIN',
                    Param::PARAM_INPUT,
                    $dtfin,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_PRODINI',
                    Param::PARAM_INPUT,
                    $prodini,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_PRODFIN',
                    Param::PARAM_INPUT,
                    $prodfin,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDREGIAO',
                    Param::PARAM_INPUT,
                    $cdregiao,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDFORNECED',
                    Param::PARAM_INPUT,
                    $cdforneced,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDMARCA',
                    Param::PARAM_INPUT,
                    $cdmarca,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CONSIST',
                    Param::PARAM_INPUT,
                    $consist,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_IDULTPREC',
                    Param::PARAM_INPUT,
                    $idultprec,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDOPERINCL',
                    Param::PARAM_INPUT,
                    $cdoperincl,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_ROWSAFFECT',
                    Param::PARAM_OUTPUT,
                    null,
                    Param::PARAM_TYPE_STR,
                    200
                )
            );

            $result = $procedure->call();
            return $result;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Executa Procedure CargDescsolCot
     *
     * @param String $cdfilial    Filial
     * @param String $nrcolepreco Número da Coleta de Preço
     * @param String $dtini       Data Inicial
     * @param String $dtfin       Data Final
     *
     * @throws \Exception
     * @return null
     */
    public static function execProcedureCargDescsolCot($cdfilial, $nrcolepreco, $dtini, $dtfin)
    {
        try {
            $procedure = new Procedure('CARG_DESCSOL_COT');
            $procedure->addParam(
                new Param(
                    'P_CDFILIAL',
                    Param::PARAM_INPUT,
                    $cdfilial,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_NRCOLEPRECO',
                    Param::PARAM_INPUT,
                    $nrcolepreco,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_DTINI',
                    Param::PARAM_INPUT,
                    $dtini,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_DTFIN',
                    Param::PARAM_INPUT,
                    $dtfin,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Executa a procedura IMPOSTO_ITEMCOL
     *
     * @param String $cdempresa   Empresa
     * @param String $cdinscesta  Inscrição Estadual
     * @param String $cdforneced  Fornecedor
     * @param String $cdfilial    Unidade
     * @param String $nrcolepreco Número coleta de preço
     * @param String $cdregiao    Região
     * @param String $cdproduto   Produto
     * @param String $cdmarca     Marca
     * @param String $dtinivalid  Dt Início Validade
     *
     * @throws \Exception
     * @return null
     */
    public static function execProcedureImpostoItemCol(
        $cdempresa, $cdinscesta, $cdforneced, $cdfilial, $nrcolepreco,
        $cdregiao, $cdproduto, $cdmarca, $dtinivalid
    ) {
        try {
            $procedure = new Procedure('IMPOSTO_ITEMCOL');
            $procedure->addParam(
                new Param(
                    'P_EMPRESA',
                    Param::PARAM_INPUT,
                    $cdempresa,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_INSCESTA',
                    Param::PARAM_INPUT,
                    $cdinscesta,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_FORNECEDOR',
                    Param::PARAM_INPUT,
                    $cdforneced,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_FILCENT',
                    Param::PARAM_INPUT,
                    $cdfilial,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_LANCCOLP',
                    Param::PARAM_INPUT,
                    $nrcolepreco,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDFORNECED',
                    Param::PARAM_INPUT,
                    $cdforneced,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDREGIAO',
                    Param::PARAM_INPUT,
                    $cdregiao,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param(
                    'P_CDPRODUTO',
                    Param::PARAM_INPUT,
                    $cdproduto,
                    Param::PARAM_TYPE_STR,
                    1
                )
            );

            $procedure->addParam(
                new Param(
                    'P_CDMARCA',
                    Param::PARAM_INPUT,
                    $cdmarca,
                    Param::PARAM_TYPE_STR,
                    1
                )
            );

            $procedure->addParam(
                new Param(
                    'P_INIVALID',
                    Param::PARAM_INPUT,
                    $dtinivalid,
                    Param::PARAM_TYPE_STR,
                    1
                )
            );

            $procedure->addParam(
                new Param(
                    'P_TRANSACAO',
                    Param::PARAM_INPUT,
                    'S',
                    Param::PARAM_TYPE_STR,
                    1
                )
            );

            return $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Executa a procedura de incluir novo fornecedor na cotação de preços
     *
     * @param String $cdfilial    Unidade
     * @param String $nrcolepreco Número da Coleta de Preço
     * @param String $cdforneced  Fornecedor
     * @param String $cdopersis   Operador
     * @param String $tregiao     Todas Regiões
     * @param String $cdforncarg  Forencedor carg
     * @param String $cdregcarg   Região carg
     *
     * @throws \Exception
     *
     * @return null
     */
    public static function execProcedureIncFornCota(
        $cdfilial, $nrcolepreco, $cdforneced, $cdopersis, $tregiao, $cdforncarg,
        $cdregcarg
    ) {
        try {
            $procedure = new Procedure('INC_FORN_COTA');
            $procedure->addParam(
                new Param(
                    'P_CDFILIAL',
                    Param::PARAM_INPUT,
                    $cdfilial,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_NRCOLEPRECO',
                    Param::PARAM_INPUT,
                    $nrcolepreco,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDFORNECED',
                    Param::PARAM_INPUT,
                    $cdforneced,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDOPERSIS',
                    Param::PARAM_INPUT,
                    $cdopersis,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_TREGIAO',
                    Param::PARAM_INPUT,
                    $tregiao,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDFORNCARG',
                    Param::PARAM_INPUT,
                    $cdforncarg,
                    Param::PARAM_TYPE_STR
                )
            );
            $procedure->addParam(
                new Param(
                    'P_CDREGCARG',
                    Param::PARAM_INPUT,
                    $cdregcarg,
                    Param::PARAM_TYPE_STR
                )
            );

            $procedure->addParam(
                new Param(
                    'P_IDERRO',
                    Param::PARAM_OUTPUT,
                    null,
                    Param::PARAM_TYPE_STR,
                    1
                )
            );

            return $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna o próximo código do número da coleta de preço
     *
     * @param String $nrorg Organização
     * @param String $cdfilial Unidade
     *
     * @throws \Exception
     * @return null
     */
    public static function getProxNrcolepreco($nrorg, $cdfilial)
    {
        try{
            $stcodigo  = 'COLETA';
            $stcodigo .= $cdfilial;
            $codigo   = Novocodigo::geraCodigo($nrorg, $stcodigo, 1);
            return $codigo;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna o código das verificações concatenadas por '_'
     *
     * @param String $verificacoes Verificações concatenadas
     *
     * @return int
     */
    public static function getConsist($verificacoes)
    {
        $valores = array();
        if (!(strpos($verificacoes, '0') == false)) {
            $valores[] = '0';
        }
        if (!(strpos($verificacoes, '1') == false)) {
            $valores[] = '1';
        }
        if (!(strpos($verificacoes, '2') == false)) {
            $valores[] = '2';
        }
        if (!(strpos($verificacoes, '3') == false)) {
            $valores[] = '3';
        }

        return implode("_", $valores);
    }

    /**
     * Retorna o item de coleta
     *
     * @param String $nrorg       Organização
     * @param String $cdfilial    Unidade
     * @param String $nrcolepreco Número da coleta de preço
     * @param String $cdregiao    Região
     * @param String $cdforneced  Fornecedor
     * @param String $cdproduto   Produto
     * @param String $cdmarca     Marca
     *
     * @return Itemcole
     * @throws \Exception
     */
    public static function getItemCole(
        $nrorg, $cdfilial, $nrcolepreco, $cdregiao, $cdforneced, $cdproduto,
        $cdmarca
    ) {
        try {
            /**
             * @var \Suprimentos\Entities\Itemcole $produtoMarca
             */
            $produtoMarca = Itemcole::getObjectByWhere(
                'NRORG = ? AND CDFILIAL = ? AND NRCOLEPRECO = ? AND CDREGIAO = ?
                AND CDFORNECED = ? AND CDPRODUTO = ? AND CDMARCA = ?',
                array($nrorg, $cdfilial, $nrcolepreco, $cdregiao, $cdforneced,
                    $cdproduto, $cdmarca)
            );
            return $produtoMarca;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna a última data de validação de qualificação
     *
     * @param String $cdfilial Unidade
     *
     * @return mixed
     * @throws \Exception
     */
    public static function getIdultidtvalq($cdfilial)
    {
        try {
            /**
             * Última data de validação de qualificação
             * @var \Geral\Entities\Parasupri $parasupri
             */
            $parasupri = Parasupri::getObjectByWhere(
                'CDFILIAL = ?', array($cdfilial)
            );
            $idultidtvalq = $parasupri->getIdultidtvalq();
            return $idultidtvalq;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna os parâmetros empresa, inscrição estadual e Dt Inicial de Validade
     * que serão usados na procedura IMPOSTO_ITEMCOL
     *
     * @param String $nrorg     Organização
     * @param String $cdproduto Produto
     *
     * @throws \Exception
     *
     * @return null
     */
    public static function getParametrosAliquota($nrorg, $cdproduto)
    {
        try {
            /**
             * @var \Fiscal\Entities\Aliquota $aliquota
             */
            $aliquota = Aliquota::getObjectByWhere(
                'NRORG = ? AND CDPRODUTO = ?',
                array($nrorg, $cdproduto)
            );
            if ($aliquota) {
                $cdempresa = $aliquota->getCdempresa();
                $cdincesta = $aliquota->getCdinscesta();
                $dtinivalid = $aliquota->getDtinivalid();
                return array($cdempresa, $cdincesta, $dtinivalid);
            } else {
                return array(null, null, null);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna Número de dias de pagamento e o valor percentual do prazo de
     * pagamento do fornecedor
     *
     * @param String $nrorg      Organização
     * @param String $cdforneced Fornecedor
     *
     * @return array
     * @throws \Exception
     */
    public static function getPrazopgtforn($nrorg, $cdforneced)
    {
        try {
            /**
             * @var \Geral\Entities\Prazopgtforn $pagamento
             */
            $pagamento = Prazopgtforn::getObjectByWhere(
                'NRORG = ? AND CDFORNECED = ?',
                array($nrorg, $cdforneced)
            );
            if ($pagamento) {
                $nrdiaspgtofo = $pagamento->getNrdiaspgtofo();
                $vrpercpgtofo = $pagamento->getVrpercpgtofo();
                return array($nrdiaspgtofo, $vrpercpgtofo);
            } else {
                return array(null, null, null);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }



    /**
     * Retorna os preços dos itens do fornecedor e região
     *
     * @param String $nrorg       Organização
     * @param String $cdfilial    Unidade
     * @param String $nrcolepreco Número da coleta de preço
     * @param String $cdforneced  Fornecedor
     * @param String $cdregiao    Região
     *
     * @return Itemcole
     * @throws \Exception
     */
    public static function getPrecosFornecedorRegiao(
        $nrorg, $cdfilial, $nrcolepreco, $cdforneced, $cdregiao
    ) {
        try {
            /**
             * Fornecedor/Região
             * @var \Suprimentos\Entities\Itemcole $fornecedorRegiao
             */
            $fornecedorRegiao = Itemcole::getObjectsByWhere(
                'NRORG = ? AND CDFILIAL = ? AND NRCOLEPRECO = ? AND CDREGIAO = ?
                AND CDFORNECED = ?',
                array($nrorg, $cdfilial, $nrcolepreco, $cdregiao, $cdforneced)
            );
            return $fornecedorRegiao;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna valores concatenados do multisearch ou outro valor
     *
     * @param Array  $campo  Conteúdo do Campo
     * @param String $codigo Código do campo
     *
     * @return string
     */
    public static function getValorMultiSearch($campo, $codigo)
    {
        if (is_array($campo)) {
            $concatenado = '_';
            foreach ($campo as $valor) {
                $concatenado .= $valor[$codigo].'_';
            }
            return $concatenado;

        } else {
            return $campo;
        }
    }

    /**
     * Exclui a região do fornecedor corrente
     *
     * @param String $nrorg       Organização
     * @param String $cdfilial    Unidade
     * @param String $nrcolepreco Número da coleta
     * @param String $cdregiao    Região
     * @param String $cdforneced  Fornecedor
     *
     * @return null
     * @throws \Exception
     */
    public static function excluiRegiaoFornecedor(
        $nrorg, $cdfilial, $nrcolepreco, $cdregiao, $cdforneced
    ) {
        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            /**
             * Região do fornecedor corrente
             * @var \Suprimentos\Entities\Itemcole $regiao
             */
            $regiao = Itemcole::getObjectsByWhere(
                'NRORG = ? AND CDFILIAL = ? AND NRCOLEPRECO = ? AND CDREGIAO = ?
                AND CDFORNECED = ?',
                array($nrorg, $cdfilial, $nrcolepreco, $cdregiao, $cdforneced)
            );

            foreach ($regiao as $item) {
                /**
                 * @var \Suprimentos\Entities\Itemcole $item
                 */
                $item->delete();
                $conn->flush();
            }

            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * Excluir região para todos fornecedores
     *
     * @param String $nrorg       Organização
     * @param String $cdfilial    Unidade
     * @param String $nrcolepreco Número da Coleta
     * @param String $cdregiao    Região
     *
     * @return null
     * @throws \Exception
     */
    public static function excluiRegiaoTodosFornecedores(
        $nrorg, $cdfilial, $nrcolepreco, $cdregiao
    ) {
        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            /**
             * Região de todos fornecedores
             * @var \Suprimentos\Entities\Itemcole $regiao
             */
            $regiao = Itemcole::getObjectsByWhere(
                'NRORG = ? AND CDFILIAL = ? AND NRCOLEPRECO = ? AND CDREGIAO = ?',
                array($nrorg, $cdfilial, $nrcolepreco, $cdregiao)
            );

            foreach ($regiao as $item) {
                /**
                 * @var \Suprimentos\Entities\Itemcole $item
                 */
                $item->delete();
                $conn->flush();
            }

            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * Exclui o fornecedor
     *
     * @param String $nrorg       Organização
     * @param String $cdfilial    Unidade
     * @param String $nrcolepreco Número da coleta
     * @param String $cdforneced  Fornecedor
     * @param String $cdregiao    Região
     *
     * @return null
     * @throws \Exception
     */
    public static function excluiFornecedorRegiao(
        $nrorg, $cdfilial, $nrcolepreco, $cdforneced, $cdregiao
    ) {
        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            /**
             * Fornecedor
             * @var \Suprimentos\Entities\Itemcole $regiao
             */
            $fornecedor = Itemcole::getObjectsByWhere(
                'NRORG = ? AND CDFILIAL = ? AND NRCOLEPRECO = ? AND CDFORNECED = ?
                 AND CDREGIAO = ?',
                array($nrorg, $cdfilial, $nrcolepreco, $cdforneced, $cdregiao)
            );

            foreach ($fornecedor as $item) {
                /**
                 * @var \Suprimentos\Entities\Itemcole $item
                 */
                $item->delete();
                $conn->flush();
            }

            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * Exclui o fornecedor de todas regiões
     *
     * @param String $nrorg       Organização
     * @param String $cdfilial    Unidade
     * @param String $nrcolepreco Número da coleta
     * @param String $cdforneced  Fornecedor
     *
     * @return null
     * @throws \Exception
     */
    public static function excluiFornecedorTodasRegioes(
        $nrorg, $cdfilial, $nrcolepreco, $cdforneced
    ) {
        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            /**
             * Fornecedor de todas regiões
             * @var \Suprimentos\Entities\Itemcole $fornecedor
             */
            $fornecedor = Itemcole::getObjectsByWhere(
                'NRORG = ? AND CDFILIAL = ? AND NRCOLEPRECO = ? AND CDFORNECED = ?',
                array($nrorg, $cdfilial, $nrcolepreco, $cdforneced)
            );
            foreach ($fornecedor as $item) {
                /**
                 * @var \Suprimentos\Entities\Itemcole $item
                 */
                $item->delete();
                $conn->flush();
            }

            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * Excluir a cotação de preços e seus itens
     *
     * @param String $nrorg       Organização
     * @param String $cdfilial    Unidade
     * @param String $nrcolepreco Número da Coleta
     *
     * @throws \Exception
     *
     * @return null
     */
    public static function excluiCotacao(
        $nrorg, $cdfilial, $nrcolepreco
    ) {
        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            CompradorLogic::validaExclusaoCotacao($nrorg);

            /**
             * Cotação de preços
             * @var Coletapr $cotacao
             */
            $cotacao = Coletapr::getObjectByWhere(
                'NRORG = ? AND CDFILIAL = ? AND NRCOLEPRECO = ?',
                array($nrorg, $cdfilial, $nrcolepreco)
            );

            if ($cotacao instanceof Coletapr) {
                $cotacao->delete();
                $conn->flush();
            }

            /**
             * @var \Suprimentos\Repositories\Itemcole $itemCole
             */
            $itemCole = ItemCole::getRepository();
            $itemCole->excluiItemColeta($nrorg, $cdfilial, $nrcolepreco);

//            /**
//             * Itens da cotação de preços
//             * @var \Suprimentos\Entities\Itemcole $regiao
//             */
//            $itemCotacao = Itemcole::getObjectsByWhere(
//                'NRORG = ? AND CDFILIAL = ? AND NRCOLEPRECO = ?',
//                array($nrorg, $cdfilial, $nrcolepreco)
//            );
//
//            if (!empty($itemCotacao)) {
//                foreach ($itemCotacao as $item) {
//                    /**
//                     * Item da cotação
//                     * @var \Suprimentos\Entities\Itemcole $item
//                     */
//                    $item->delete();
//                    $conn->flush();
//                }
//            }

            $conn->commit();
        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * Insere Produto/Marca
     *
     * @param String $nrorg        Organização
     * @param String $cdfilial     Unidade
     * @param String $nrcolepreco  Número da Coleta de preço
     * @param String $cdregiao     Região
     * @param String $cdforneced   Fornecedor
     * @param String $cdproduto    Produto
     * @param String $cdmarca      Marca
     * @param String $vrprecuniitm Preço
     * @param String $dtinivalitm  Data Inicial
     * @param String $dtfinvalitm  Data Final
     * @param String $cdoperador   Operador
     * @param String $dtatual      Data de hoje
     *
     * @throws \Exception
     * @return null
     */
    public static function insereProdutoMarca(
        $nrorg, $cdfilial, $nrcolepreco, $cdregiao, $cdforneced, $cdproduto,
        $cdmarca, $vrprecuniitm, $dtinivalitm, $dtfinvalitm, $cdoperador, $dtatual
    ) {
        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            /**
             * Produto/Marca
             * @var \Suprimentos\Entities\Itemcole $produtoMarca
             */
            $produtoMarca = new Itemcole();

            $dtinivalitm = DateUtil::truncateString($dtinivalitm);
            $dtinivalitm = DateUtil::getDataDeString($dtinivalitm);
            $dtfinvalitm = DateUtil::truncateString($dtfinvalitm);
            $dtfinvalitm = DateUtil::getDataDeString($dtfinvalitm);

            $produtoMarca->setNrorg($nrorg);
            $produtoMarca->setCdfilial($cdfilial);
            $produtoMarca->setNrcolepreco($nrcolepreco);
            $produtoMarca->setCdregiao($cdregiao);
            $produtoMarca->setCdforneced($cdforneced);
            $produtoMarca->setCdproduto($cdproduto);
            $produtoMarca->setCdmarca($cdmarca);
            $produtoMarca->setVrprecuniitm($vrprecuniitm);
            $produtoMarca->setDtinivalitm($dtinivalitm);
            $produtoMarca->setDtfinvalitm($dtfinvalitm);
            $produtoMarca->setIdtipfreitm('C');
            $produtoMarca->setIdprecalt('N');
            $produtoMarca->setIditmlisprec('N');
            $produtoMarca->setIditmcotprec('S');
            $produtoMarca->setIdaturegiinc('S');
            $produtoMarca->setIdatuforninc('S');
            $produtoMarca->setIditvaldiflista('N');

            //auditoria
            $produtoMarca->setCdoperincl($cdoperador);
            $produtoMarca->setCdoperinclusao($cdoperador);
            $produtoMarca->setCdoperultatu($cdoperador);
            $produtoMarca->setDtinclitmcol($dtatual);
            $produtoMarca->setDtinclusao($dtatual);
            $produtoMarca->setDtultatu($dtatual);
            $produtoMarca->setNrorgultatu($nrorg);
            $produtoMarca->setNrorginclusao($nrorg);

            $produtoMarca->persist();

            $conn->flush();
            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * Insere Fornecedor
     *
     * @param String                         $nrorg            Organização
     * @param String                         $cdfilial         Unidade
     * @param String                         $nrcolepreco      Número da Coleta de Preço
     * @param String                         $cdforneced       Fornecedor
     * @param \Suprimentos\Entities\Itemcole $regiaoFornecedor Região-Fornecedor
     * @param String                         $cdoperador       Operador
     * @param String                         $dtatual          Data atual
     *
     * @throws \Exception
     * @return bool
     */
    public static function insereFornecedor(
        $nrorg, $cdfilial, $nrcolepreco, $cdforneced, Itemcole $regiaoFornecedor,
        $cdoperador, $dtatual
    ) {
        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            /**
             * Fornecedor
             * @var \Suprimentos\Entities\Itemcole $fornecedor
             */
            $fornecedor = new Itemcole();

            $fornecedor->setNrorg($nrorg);
            $fornecedor->setCdfilial($cdfilial);
            $fornecedor->setNrcolepreco($nrcolepreco);
            $fornecedor->setCdforneced($cdforneced);

            $fornecedor->setCdregiao($regiaoFornecedor->getCdregiao());
            $fornecedor->setCdproduto($regiaoFornecedor->getCdproduto());
            $fornecedor->setCdmarca($regiaoFornecedor->getCdmarca());
            $fornecedor->setCdtipcob($regiaoFornecedor->getCdtipcob());
            $fornecedor->setCdformpgto($regiaoFornecedor->getCdformpgto());
            $fornecedor->setCdsegmarccol($regiaoFornecedor->getCdsegmarccol());
            $fornecedor->setDtinivalitm($regiaoFornecedor->getDtinivalitm());
            $fornecedor->setDtfinvalitm($regiaoFornecedor->getDtfinvalitm());
            $fornecedor->setVrprecuniitm($regiaoFornecedor->getVrprecuniitm());
            $fornecedor->setVrprececoitm($regiaoFornecedor->getVrprececoitm());
            $fornecedor->setQtacmpitmcol($regiaoFornecedor->getQtacmpitmcol());
            $fornecedor->setQtpreventitm($regiaoFornecedor->getQtpreventitm());
            $fornecedor->setIdtipdescol($regiaoFornecedor->getIdtipdescol());
            $fornecedor->setVrperdesitm($regiaoFornecedor->getVrperdesitm());
            $fornecedor->setIdtipfreitm($regiaoFornecedor->getIdtipfreitm());
            $fornecedor->setIdtipfornitm($regiaoFornecedor->getIdtipfornitm());
            $fornecedor->setIdprecalt($regiaoFornecedor->getIdprecalt());
            $fornecedor->setIditmlisprec($regiaoFornecedor->getIditmlisprec());
            $fornecedor->setIditmcotprec($regiaoFornecedor->getIditmcotprec());
            $fornecedor->setNrparcpgtcol($regiaoFornecedor->getNrparcpgtcol());
            $fornecedor->setNrdiapgfxcol($regiaoFornecedor->getNrdiapgfxcol());
            $fornecedor->setIdutdiapgcol($regiaoFornecedor->getIdutdiapgcol());
            $fornecedor->setDspzopgtconc($regiaoFornecedor->getDspzopgtconc());
            $fornecedor->setDsobsitmcol($regiaoFornecedor->getDsobsitmcol());
            $fornecedor->setVrprececoitm($regiaoFornecedor->getVrprececoitm());
            $fornecedor->setNrseqprelist($regiaoFornecedor->getNrseqprelist());
            $fornecedor->setNrcoleorigic($regiaoFornecedor->getNrcoleorigic());
            $fornecedor->setIdaturegiinc($regiaoFornecedor->getIdaturegiinc());
            $fornecedor->setIdaturegialt($regiaoFornecedor->getIdaturegialt());
            $fornecedor->setIdatuforninc($regiaoFornecedor->getIdatuforninc());
            $fornecedor->setVrprecunorig($regiaoFornecedor->getVrprecunorig());
            $fornecedor->setVrprececorig($regiaoFornecedor->getVrprececorig());
            $fornecedor->setVrprecpiorig($regiaoFornecedor->getVrprecpiorig());
            $fornecedor->setIditecolealt($regiaoFornecedor->getIditecolealt());
            $fornecedor->setNrpedidocol($regiaoFornecedor->getNrpedidocol());
            $fornecedor->setIditvaldiflista($regiaoFornecedor->getIditvaldiflista());

            //auditoria
            $fornecedor->setCdoperincl($cdoperador);
            $fornecedor->setCdoperinclusao($cdoperador);
            $fornecedor->setCdoperultatu($cdoperador);
            $fornecedor->setDtinclitmcol($dtatual);
            $fornecedor->setDtinclusao($dtatual);
            $fornecedor->setDtultatu($dtatual);
            $fornecedor->setNrorgultatu($nrorg);
            $fornecedor->setNrorginclusao($nrorg);

            $fornecedor->persist();

            $conn->flush();
            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * Insere Fornecedor
     *
     * @param String                         $nrorg            Organização
     * @param String                         $cdfilial         Unidade
     * @param String                         $nrcolepreco      Número da Coleta de Preço
     * @param String                         $cdregiao         Região
     * @param \Suprimentos\Entities\Itemcole $regiaoFornecedor Região-Fornecedor
     * @param String                         $cdoperador       Operador
     * @param String                         $dtatual          Data atual
     *
     * @throws \Exception
     *
     * @return bool
     */
    public static function insereRegiao(
        $nrorg, $cdfilial, $nrcolepreco, $cdregiao, Itemcole $regiaoFornecedor,
        $cdoperador, $dtatual
    ) {
        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            /**
             * Fornecedor
             * @var \Suprimentos\Entities\Itemcole $fornecedor
             */
            $regiao = new Itemcole();

            $regiao->setNrorg($nrorg);
            $regiao->setCdfilial($cdfilial);
            $regiao->setNrcolepreco($nrcolepreco);
            $regiao->setCdregiao($cdregiao);

            $regiao->setCdforneced($regiaoFornecedor->getCdforneced());
            $regiao->setCdproduto($regiaoFornecedor->getCdproduto());
            $regiao->setCdmarca($regiaoFornecedor->getCdmarca());
            $regiao->setCdtipcob($regiaoFornecedor->getCdtipcob());
            $regiao->setCdformpgto($regiaoFornecedor->getCdformpgto());
            $regiao->setCdsegmarccol($regiaoFornecedor->getCdsegmarccol());
            $regiao->setDtinivalitm($regiaoFornecedor->getDtinivalitm());
            $regiao->setDtfinvalitm($regiaoFornecedor->getDtfinvalitm());
            $regiao->setVrprecuniitm($regiaoFornecedor->getVrprecuniitm());
            $regiao->setVrprececoitm($regiaoFornecedor->getVrprececoitm());
            $regiao->setQtacmpitmcol($regiaoFornecedor->getQtacmpitmcol());
            $regiao->setQtpreventitm($regiaoFornecedor->getQtpreventitm());
            $regiao->setIdtipdescol($regiaoFornecedor->getIdtipdescol());
            $regiao->setVrperdesitm($regiaoFornecedor->getVrperdesitm());
            $regiao->setIdtipfreitm($regiaoFornecedor->getIdtipfreitm());
            $regiao->setIdtipfornitm($regiaoFornecedor->getIdtipfornitm());
            $regiao->setIdprecalt($regiaoFornecedor->getIdprecalt());
            $regiao->setIditmlisprec($regiaoFornecedor->getIditmlisprec());
            $regiao->setIditmcotprec($regiaoFornecedor->getIditmcotprec());
            $regiao->setNrparcpgtcol($regiaoFornecedor->getNrparcpgtcol());
            $regiao->setNrdiapgfxcol($regiaoFornecedor->getNrdiapgfxcol());
            $regiao->setIdutdiapgcol($regiaoFornecedor->getIdutdiapgcol());
            $regiao->setDspzopgtconc($regiaoFornecedor->getDspzopgtconc());
            $regiao->setDsobsitmcol($regiaoFornecedor->getDsobsitmcol());
            $regiao->setVrprececoitm($regiaoFornecedor->getVrprececoitm());
            $regiao->setNrseqprelist($regiaoFornecedor->getNrseqprelist());
            $regiao->setNrcoleorigic($regiaoFornecedor->getNrcoleorigic());
            $regiao->setIdaturegiinc($regiaoFornecedor->getIdaturegiinc());
            $regiao->setIdaturegialt($regiaoFornecedor->getIdaturegialt());
            $regiao->setIdatuforninc($regiaoFornecedor->getIdatuforninc());
            $regiao->setVrprecunorig($regiaoFornecedor->getVrprecunorig());
            $regiao->setVrprececorig($regiaoFornecedor->getVrprececorig());
            $regiao->setVrprecpiorig($regiaoFornecedor->getVrprecpiorig());
            $regiao->setIditecolealt($regiaoFornecedor->getIditecolealt());
            $regiao->setNrpedidocol($regiaoFornecedor->getNrpedidocol());
            $regiao->setIditvaldiflista($regiaoFornecedor->getIditvaldiflista());

            //auditoria
            $regiao->setCdoperincl($cdoperador);
            $regiao->setCdoperinclusao($cdoperador);
            $regiao->setCdoperultatu($cdoperador);
            $regiao->setDtinclitmcol($dtatual);
            $regiao->setDtinclusao($dtatual);
            $regiao->setDtultatu($dtatual);
            $regiao->setNrorgultatu($nrorg);
            $regiao->setNrorginclusao($nrorg);

            $regiao->persist();

            $conn->flush();
            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $nrcolepreco
     * @param $cdregiao
     * @param $cdforneced
     * @param $cdproduto
     * @param $cdmarca
     * @param $vrprecuniitm
     * @param $dtinivalitm
     * @param $dtfinvalitm
     * @param $cdoperador
     * @param $dtatual
     *
     * @return bool
     * @throws \Exception
     */
    public static function insereItemCole(
        $nrorg, $cdfilial, $nrcolepreco, $cdregiao, $cdforneced, $cdproduto,
        $cdmarca, $vrprecuniitm, $dtinivalitm, $dtfinvalitm, $cdoperador, $dtatual
    ) {
        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            /**
             * Item da coleta de preço
             * @var \Suprimentos\Entities\Itemcole $itemCole
             */
            $itemCole = new Itemcole();

            $dtinivalitm = DateUtil::truncateString($dtinivalitm);
            $dtinivalitm = DateUtil::getDataDeString($dtinivalitm);
            $dtfinvalitm = DateUtil::truncateString($dtfinvalitm);
            $dtfinvalitm = DateUtil::getDataDeString($dtfinvalitm);

            $itemCole->setNrorg($nrorg);
            $itemCole->setCdfilial($cdfilial);
            $itemCole->setNrcolepreco($nrcolepreco);
            $itemCole->setCdregiao($cdregiao);
            $itemCole->setCdforneced($cdforneced);
            $itemCole->setCdproduto($cdproduto);
            $itemCole->setCdmarca($cdmarca);
            $itemCole->setVrprecuniitm($vrprecuniitm);
            $itemCole->setDtinivalitm($dtinivalitm);
            $itemCole->setDtfinvalitm($dtfinvalitm);
            $itemCole->setIdtipfreitm('C');
            $itemCole->setIdprecalt('N');
            $itemCole->setIditmlisprec('N');
            $itemCole->setIditmcotprec('S');
            $itemCole->setIdaturegiinc('S');
            $itemCole->setIdatuforninc('S');
            $itemCole->setIditvaldiflista('N');

            //auditoria
            $itemCole->setCdoperincl($cdoperador);
            $itemCole->setCdoperinclusao($cdoperador);
            $itemCole->setCdoperultatu($cdoperador);
            $itemCole->setDtinclitmcol($dtatual);
            $itemCole->setDtinclusao($dtatual);
            $itemCole->setDtultatu($dtatual);
            $itemCole->setNrorgultatu($nrorg);
            $itemCole->setNrorginclusao($nrorg);

            $itemCole->persist();

            $conn->flush();
            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * Insere prazo pagamento do item de coleta
     *
     * @param String $nrorg        Organização
     * @param String $nrdiaspgtofo Número dias pagamento
     * @param String $vrpercpgtofo Valor percentual pagamento
     * @param String $cdfilial     Unidade
     * @param String $nrcolepreco  Número da coleta de preço
     * @param String $cdregiao     Região
     * @param String $cdforneced   Fornecedor
     * @param String $cdproduto    Produto
     * @param String $cdmarca      Marca
     * @param String $cdoperador   Operador
     * @param String $dtatual      Data Atual
     *
     * @throws \Exception
     * @return bool
     */
    public static function inserePzopgtitmcol(
        $nrorg, $nrdiaspgtofo, $vrpercpgtofo, $cdfilial, $nrcolepreco,
        $cdregiao, $cdforneced, $cdproduto, $cdmarca, $cdoperador, $dtatual
    ) {
        $conn = Registry::getEntityManager();
        $conn->beginTransaction();
        try {
            /**
             * Produto/Marca
             *
             * @var \Suprimentos\Entities\Itemcole $produtoMarca
             */
            $pzopgtitmcol = new Pzopgtitmcol();

            $pzopgtitmcol->setNrdiaspgtcol($nrdiaspgtofo);
            $pzopgtitmcol->setNrorg($nrorg);
            $pzopgtitmcol->setCdfilial($cdfilial);
            $pzopgtitmcol->setNrcolepreco($nrcolepreco);
            $pzopgtitmcol->setCdmarca($cdmarca);
            $pzopgtitmcol->setCdregiao($cdregiao);
            $pzopgtitmcol->setCdforneced($cdforneced);
            $pzopgtitmcol->setCdproduto($cdproduto);
            $pzopgtitmcol->setVrpercpgtcol($vrpercpgtofo);
            $pzopgtitmcol->setDtinclusao($dtatual);
            $pzopgtitmcol->setNrorginclusao($nrorg);
            $pzopgtitmcol->setCdoperinclusao($cdoperador);
            $pzopgtitmcol->setCdoperultatu($cdoperador);
            $pzopgtitmcol->setDtultatu($dtatual);

            $pzopgtitmcol->persist();

            $conn->flush();
            $conn->commit();
            return true;

        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * Atualiza o preço unitário do produto
     *
     * @param String $nrorg        Organização
     * @param String $cdfilial     Unidade
     * @param String $nrcolepreco  Número da coleta de preço
     * @param String $cdregiao     Região
     * @param String $cdforneced   Fornecedor
     * @param String $cdproduto    Produto
     * @param String $cdmarca      Marca
     * @param String $vrprecuniitm Preço Unitário
     *
     * @throws \Exception
     * @return Itemcole
     */
    public static function atualizaVrprecuniitm(
        $nrorg, $cdfilial, $nrcolepreco, $cdregiao, $cdforneced, $cdproduto,
        $cdmarca, $vrprecuniitm
    ) {

        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            /**
             * @var \Suprimentos\Entities\Itemcole $produto
             */
            $produto = Itemcole::getObjectByWhere(
                'NRORG = ? AND CDFILIAL = ? AND NRCOLEPRECO = ? AND CDREGIAO = ?
                AND CDFORNECED = ? AND CDPRODUTO = ? AND CDMARCA = ?',
                array($nrorg, $cdfilial, $nrcolepreco, $cdregiao, $cdforneced,
                    $cdproduto, $cdmarca)
            );
            if ($produto) {
                $produto->setVrprecuniitm($vrprecuniitm);
            }

            $produto->persist();

            $conn->flush();
            $conn->commit();
            return true;

        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * Atualiza o preço unitário do produto para outras regiões
     *
     * @param String $nrorg        Organização
     * @param String $cdfilial     Unidade
     * @param String $nrcolepreco  Número da coleta de preço
     * @param String $cdforneced   Fornecedor
     * @param String $cdproduto    Produto
     * @param String $cdmarca      Marca
     * @param String $vrprecuniitm Preço Unitário
     *
     * @throws \Exception
     * @return Itemcole
     */
    public static function atualizaVrprecuniitmRegioes(
        $nrorg, $cdfilial, $nrcolepreco, $cdforneced, $cdproduto,
        $cdmarca, $vrprecuniitm
    ) {
        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            /**
             * @var \Suprimentos\Entities\Itemcole $produto
             */
            $produto = Itemcole::getObjectsByWhere(
                'NRORG = ? AND CDFILIAL = ? AND NRCOLEPRECO = ?
                AND CDFORNECED = ? AND CDPRODUTO = ? AND CDMARCA = ?',
                array($nrorg, $cdfilial, $nrcolepreco, $cdforneced,
                    $cdproduto, $cdmarca)
            );
            if ($produto) {
                foreach ($produto as $item) {
                    /**
                     * @var \Suprimentos\Entities\Itemcole $item
                     */
                    $item->setVrprecuniitm($vrprecuniitm);
                    $item->persist();
                }
            }

            $conn->flush();
            $conn->commit();
            return true;

        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * Atualiza o preço unitário do produto para outros fornecedores
     *
     * @param String $nrorg        Organização
     * @param String $cdfilial     Unidade
     * @param String $nrcolepreco  Número da coleta de preço
     * @param String $cdproduto    Produto
     * @param String $cdmarca      Marca
     * @param String $vrprecuniitm Preço Unitário
     *
     * @throws \Exception
     *
     * @return Itemcole
     */
    public static function atualizaVrprecuniitmFornecedoress(
        $nrorg, $cdfilial, $nrcolepreco, $cdproduto,
        $cdmarca, $vrprecuniitm
    ) {
        $conn = Registry::getEntityManager();
        $conn->beginTransaction();

        try {
            /**
             * @var \Suprimentos\Entities\Itemcole $produto
             */
            $produto = Itemcole::getObjectsByWhere(
                'NRORG = ? AND CDFILIAL = ? AND NRCOLEPRECO = ?
                AND CDPRODUTO = ? AND CDMARCA = ?',
                array($nrorg, $cdfilial, $nrcolepreco, $cdproduto, $cdmarca)
            );
            if ($produto) {
                foreach ($produto as $item) {
                    /**
                     * @var \Suprimentos\Entities\Itemcole $item
                     */
                    $item->setVrprecuniitm($vrprecuniitm);
                    $item->persist();
                }
            }

            $conn->flush();
            $conn->commit();
            return true;

        } catch (\Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    /**
     * Verifica Qualificação por Fornecedor
     *
     * @param String $nrorg      Organização
     * @param String $cdforneced Fornecedor
     *
     * @throws \Exception
     */
    public static function validaQualificacao($nrorg, $cdforneced)
    {
        try {
            /** @var \Geral\Entities\Fornecedor $obj_Fornecedor */
            $obj_Fornecedor = Fornecedor::getObjectByWhere(
                "NRORG = ? AND CDFORNECED = ?", array($nrorg, $cdforneced)
            );
            if (is_a($obj_Fornecedor, '\Geral\Entities\Fornecedor')) {
                /** @var \Geral\Entities\Fornecedor $cdqualif */
                $cdqualif = $obj_Fornecedor->getCdqualif();
                if (empty($cdqualif)) {
                        ExceptionCotacao::erroFornecedorSemQualificacao();
                }
            }
        } catch (\Exception $e) {
            throw  $e;
        }
    }

    /**
     * Verifica se existe produto(s) ativo(s) no intervalo
     *
     * @param String $nrorg
     * @param String $idprodativ
     * @param String $prodini
     * @param String $prodfin
     * @param String $tproduto
     *
     * @throws \Exception
     */
    public static function validaProdutoAtivo(
        $nrorg, $idprodativ, $prodini, $prodfin, $tproduto
    ){
        try {
            $auxproduto = 'T';
            /**
             * @var \Geral\Entities\Produto $produto
             */
            $produto = Produto::getObjectsByWhere(
                'NRORG = ? AND IDPRODATIVPR = ? AND ((CDARVPROD BETWEEN ? AND ?) OR (? = ?))',
                array($nrorg, $idprodativ, $prodini, $prodfin, $tproduto, $auxproduto)
            );

            if (empty($produto)) {
                ExceptionCotacao::erroProdutoInativo();
            }

        } catch (\Exception $e) {
            throw  $e;
        }
    }

    /**
     * Verificar Produto por Comprador
     *
     * @param String $nrorg Organizacao
     * @param String $comprador Comprador
     * @param String $cdprodini Produto Inicial
     *
     * @throws \Exception
     */
    public static function validaProdutoPorComprador($nrorg, $comprador, $cdprodini){
        /**
         * @var \Suprimentos\Repositories\Prodcomp $produto
         */
        $produto = Prodcomp::getRepository();
        $retorno = $produto->verificaProdutoComprador(
            $nrorg, $comprador, $cdprodini
        );

        try{
            if (!$retorno) {
                ExceptionCotacao::erroProdutoSemCadastro();
            }
        } catch (\Exception $e) {
            throw  $e;
        }
    }
}